#include <u.h>
#include <libc.h>
#include <bio.h>
#include "dat.h"
#include "fns.h"

Biobuf Out;
int Debug;

void
upd(State *st, char *val)
{
	if(val == nil)
		val = "";
	if(strcmp(val, st->val) == 0)
		return;
	snprint(st->val, sizeof(st->val), "%s", val);
	Bprint(&Out, "%s %q\n", st->nam, val);
}


int
httphdr(Sess *s, char *field, char *val, int len)
{
	int n, fd;
	char buf[128];

	if(val && len > 0)
		*val = 0;

	snprint(buf, sizeof(buf), "%s/%d/%s", s->webfs, s->conid, field);

	while((fd = open(buf, OREAD)) < 0){
		if(time(nil) - s->contime > 4)
			return -1;

fprint(2, "%s: %r, retrying...\n", argv0, buf);
		/*
		 * though the TCP session is established,
		 * a slow web server may not have sent any
		 * data yet so we we may have no headers
		 * from webfs...
		 */
		sleep(100);
	}
	n = read(fd, val, len-1);
	close(fd);

	if(n < 0)
		return -1;
	val[n] = 0;
	return 0;
}

int
httpredial(Sess *s, char *url, char *etag, int netag)
{
	int e, n, fd;
	char buf[128], *a[8], err[ERRMAX];

	if(Debug)
		fprint(2, "dialing %q\n", url);

	snprint(buf, sizeof buf, "%s/%d/ctl", s->webfs, s->conid);
	if((fd = open(buf, OWRITE)) < 0){
		werrstr("cannot open another HTTP session - %r");
		return -1;
	}
	fprint(fd, "url %s", url);
	if(etag && *etag)
		fprint(fd, "headers If-None-Match: %s", etag);
	close(fd);

	snprint(buf, sizeof buf, "%s/%d/body", s->webfs, s->conid);
	fd = open(buf, OREAD);

	if(fd < 0){
		rerrstr(err, sizeof(err));
		if(Debug)
			fprint(2, "connect failed err=%s\n", err);

		if((n = tokenize(err, a, nelem(a))) >= 3){
			e = atoi(a[1]);
			if(e == 0 && strcmp(a[n-1], "interrupted") == 0)
				werrstr("timeout on connect");
		}
		return -1;
	}

	if(etag && *etag && netag > 0)
		if(httphdr(s, "etag", etag, netag) < 0)
			*etag = 0;

	s->contime = time(nil);
	return fd;
}

int
notmodified(void)
{
	char *a[4], err[ERRMAX];

	rerrstr(err, sizeof(err));
	if(tokenize(err, a, nelem(a)) < 2)
		return 0;
	if(atoi(a[1]) != 304)
		return 0;
	return 1;
}

static void
usage(void)
{
	fprint(2, "usage: %s [-w webfs-dir] [-opts] http://...\n", argv0);
	exits("usage");
}

void
main(int argc, char *argv[])
{
	int n, net;
	char buf[128];
	Sess session;

	session.webfs = "/mnt/web";
	ARGBEGIN{
	case 'd':
		Debug++;
		break;
	case 'w':
		session.webfs = EARGF(usage());
		break;
	default:
		usage();
	}ARGEND;

	if(argc != 1)
		usage();

	srand(getpid() ^ time(nil));
	quotefmtinstall();
	Binit(&Out, 1, OWRITE);

	snprint(buf, sizeof buf, "%s/clone", session.webfs);
	if((net = open(buf, ORDWR)) < 0)
		sysfatal("%s cannot open - %r", buf);

	if((n = read(net, buf, sizeof buf-1)) < 0)
		sysfatal("%s cannot read - %r", buf);
	if(n == 0)
		sysfatal("%s is empty - %r", buf);
	buf[n] = '\0';
	session.conid = atoi(buf);

	poll(&session, argv[0]);

	close(net);
	exits(nil);
}
