#include <u.h>
#include <libc.h>
#include <bio.h>
#include <ctype.h>
#include "dat.h"
#include "fns.h"

/*
 * Beware:
 * 	missing or spurious carriage returns and linefeeds,
 * 		particularly at the end of file.
 * 	The NumberOfEntries may be missing and should not be relied on.
 * 	out of order entries, must insert into the correct place in the list
 */
typedef struct Aux Aux;
struct Aux {
	int slot;
};

static char *
getval(char *s)
{
	char *p, *e;

	for(p = s; *p && *p != '='; p++)
		continue;
	if(*p != '='){
		if(dbg("pls"))
			fprint(2, "pls.getval: %q, no '=' found", s);
		return nil;
	}
	p++;

	while(isspace(*p))
		p++;

	if(*p == '"' || *p == '\''){
		for(e = p; *e && *e != *p; e++)
			continue;
		if(*e != 0){
			if(dbg("pls"))
				fprint(2, "pls.getval: %q, unmatched quote", s);
			return nil;
		}
		*e = 0;
		p++;
	}
	return p;
}

static Chunk *
lookup(Chunk **root, char *s)
{
	int n;
	Aux *a;
	Chunk *ch, **prev;

	n = atoi(s);
	for(ch = *root; ch; ch = ch->next){
		a = ch->aux;
		if(a->slot == n)
			break;
	}

	if(ch == nil){
		/* NB: insert in correct place in sorted list */
		prev = root;
		for(ch = *root; ch; ch = ch->next){
			a = ch->aux;
			if(a->slot > n)
				break;
			prev = &ch->next;
		}
		ch = emalloc(sizeof(Chunk));
		a = emalloc(sizeof(Aux));
		ch->aux = a;
		a->slot = n;

		ch->next = *prev;
		*prev = ch;
	}

	return ch;	
}


Playlist *
pls(char *url, int fd)
{
	Chunk *ch;
	Biobuf bin;
	Playlist *pl;
	char *p, *base, *rec, buf[1024];

	base = baseurl(url);
	Binit(&bin, fd, OREAD);

	pl = emalloc(sizeof(Playlist));
	pl->isaudio = 1;

	while(Bgetline(&bin, buf, sizeof(buf)) != -1){

		rec = skipws(buf);
		if(dbg("pls"))
			fprint(2, "pls %q\n", rec);
	
		if(cistrncmp(rec, "Title", 5) == 0){
			ch = lookup(&pl->chunks, rec+5);
			p = getval(rec+5);
			if(p == nil)
				break;
			parsetitle(ch, p);
		}
		if(cistrncmp(rec, "File", 4) == 0){
			ch = lookup(&pl->chunks, rec+4);
			p = getval(rec+4);
			if(p == nil)
				break;
			ch->url = absurl(base, p);
		}
		if(cistrncmp(rec, "Length", 6) == 0){
			ch = lookup(&pl->chunks, rec+6);
			p = getval(rec+6);
			if(p == nil)
				break;
			ch->secs = strtod(p, nil);

			if(ch->secs == -1)
				pl->live = 1;
		}
	}
	Bterm(&bin);
	return pl;
}
