#include <u.h>
#include <libc.h>
#include <bio.h>
#include <xml.h>
#include "dat.h"
#include "fns.h"

/*
 * If we fetch http://www.abc.net.au/triplej/includes/nowon_radio.xml
 * we can extract the show and presenter names but it just seems like
 * its not worth the effort, particularly given maintence overhead of
 * the station list.
 */

static State Album = { "album",	"" };
static State Title = { "title",	"" };
static State Cover = { "cover",	"" };
static State Artist = { "artist",	"" };
static State Length = { "length",	"" };

/*
 * http://www.abc.net.au/triplej/feeds/playout/triplej_sydney_playout.xml
 *
 * <?xml version='1.0' encoding='utf-8' standalone='yes'?>
 * <abcmusic_playout>
 *     <channel>
 *         triplej-sydney-history2
 *     </channel>
 *     <publishtime>
 *         8.12pm
 *     </publishtime>
 *     <items>
 *         <item>
 *             <playing>
 *                 now
 *             </playing>
 *             <title>
 *                 Dynamite {Ft. Allday}
 *             </title>
 *             <trackid/>
 *             <playedtime>
 *                 2015-04-21 20:12:10Z
 *             </playedtime>
 *             <publisher/>
 *             <datecopyrighted/>
 *             <duration>
 *                 00:03:55
 *             </duration>
 *             <aust/>
 *             <tracknote/>
 *             <tracklink/>
 *             <show/>
 *             <talent/>
 *             <album>
 *                 <albumname>
 *                     Dynamite {Single}
 *                 </albumname>
 *                 <albumid>
 *                     53406
 *                 </albumid>
 *                 <albumimage>
 *                     http://www.abc.net.au/triplej/albums/53406/covers/100.jpg
 *                 </albumimage>
 *             </album>
 *             <artist>
 *                 <artistname>
 *                     Asta
 *                 </artistname>
 *                 <artistid/>
 *                 <artistnote/>
 *                 <artistlink/>
 *             </artist>
 *         </item>
 *         <item>
 *             <playing>
 *                 past
 *             </playing>
 *             <title>
 *                 Forces {Ft. Airling}
 *             </title>
 *             <trackid/>
 *             <playedtime>
 *                 2015-04-21 20:07:56Z
 *             </playedtime>
 *             <publisher/>
 *             <datecopyrighted/>
 *             <duration>
 *                 00:04:07
 *             </duration>
 *             <aust/>
 *             <tracknote/>
 *             <tracklink/>
 *             <show/>
 *             <talent/>
 *             <album>
 *                 <albumname>
 *                     Forces {Single}
 *                 </albumname>
 *                 <albumid>
 *                     53472
 *                 </albumid>
 *                 <albumimage>
 *                     http://www.abc.net.au/triplej/albums/53472/covers/100.jpg
 *                 </albumimage>
 *             </album>
 *             <artist>
 *                 <artistname>
 *                     Japanese Wallpaper
 *                 </artistname>
 *                 <artistid/>
 *                 <artistnote/>
 *                 <artistlink/>
 *             </artist>
 *         </item>
 *     </items>
 * </abcmusic_playout>
 */

static char *
tosecs(char *buf)
{
	char *a[4];
	static char ret[8];

	if(buf == nil)
		return nil;
	if(gettokens(buf, a, nelem(a), ": \t\n") != 3)
		return nil;
	snprint(ret, sizeof(ret), "%ld", 
		atol(a[0])*60*60 + atol(a[1])*60 + atol(a[2])); 
	return ret;
}
	
void
poll(Sess *s, char *url)
{
	int fd;
	Xml *xp;
	Elem *ep;
	long msec;
	char etag[1024];

	*etag = 0;
	msec = 15*1000;
	for(;;){
		if((fd = httpredial(s, url, etag, sizeof(etag))) < 0){
			if(! notmodified())
				fprint(2, "%s: %s httpredial failed, %r\n", argv0, url);
			goto again;
		}

		if((xp = xml_parse(fd, 1024*10, Fcrushwhite|Fstripnamespace)) == nil){
			fprint(2, "%s: %s XML parse failed\n", argv0, url);
			close(fd);
			goto again;
		}
		close(fd);

		if((ep = xml_find(xp, xp->root, "abcmusic_playout/items/item/title")) != nil)
			upd(&Title, ep->pcdata);

		if((ep = xml_find(xp, xp->root, "abcmusic_playout/items/item/artist/artistname")) != nil)
			upd(&Artist, ep->pcdata);

		if((ep = xml_find(xp, xp->root, "abcmusic_playout/items/item/album/albumname")) != nil)
			upd(&Album, ep->pcdata);

		if((ep = xml_find(xp, xp->root, "abcmusic_playout/items/item/album/albumimage")) != nil)
			upd(&Cover, ep->pcdata);

		if((ep = xml_find(xp, xp->root, "abcmusic_playout/items/item/duration")) != nil)
			upd(&Length, tosecs(ep->pcdata));

		xml_free(xp);

		Bflush(&Out);
again:
		sleep(msec/2 + lnrand(msec));
	}
}

