#include <u.h>
#include <libc.h>
#include <bio.h>
#include <json.h>
#include "dat.h"
#include "fns.h"

static State Station = { "station",	"" };
static State Show = { "show",	"" };
static State Title = { "title",	"" };
static State Artist = { "artist",	"" };
static State Disp = { "disp",	"" };
static State Length = { "length",	"" };
static State Cover = { "cover",	"" };
static State Preview = { "preview",	"" };
static State Presenter = { "presenter",	"" };
static State Label = { "label",	"" };

void
poll(Sess *s, char *url)
{
	int fd;
	Jvalue *j;
	char *p, etag[1024];
	long msec, minms, maxms;

	*etag = 0;
	minms = 20000;
	maxms = 25000;
	for(;;){
		if((fd = httpredial(s, url, etag, sizeof(etag))) < 0){
			if(! notmodified())
				fprint(2, "%s: %s httpredial failed, %r\n", argv0, url);
			if(Debug)
				fprint(2, "%s unchanged\n", argv0);
			goto again;
		}
		if((j = json_parse(fd, "")) == nil){
			fprint(2, "%s: %s json parse failed, %r\n", argv0, url);
			close(fd);
			goto again;
		}
		close(fd);

		upd(&Station, json_strval(j, "message.service.name"));
		upd(&Label, json_strval(j, "message.label"));
		upd(&Length, json_strval(j, "message.duration"));
		upd(&Title, json_strval(j, "message.trackExtended.title"));
		upd(&Artist, json_strval(j, "message.trackExtended.artist"));
		upd(&Cover, json_strval(j, "message.trackExtended.image_default"));
		upd(&Show, json_strval(j, "message.programme.brand_title"));
		upd(&Disp, json_strval(j, "message.programme.displayTitles"));
		upd(&Presenter, json_strval(j, "message.programme.image.url"));

		p = json_strval(j, "message.trackExtended.snippet_url");
		if(p && *p && strcmp(p, "false") != 0)
			upd(&Preview, p);
		else
			upd(&Preview, nil);


		if((p = json_strval(j, "requestMaxMilliseconds")) != nil)
			maxms = atol(p);
		if((p = json_strval(j, "requestMinMilliseconds")) != nil)
			minms = atol(p);
		json_free(j);
		Bflush(&Out);
again:
		msec = minms + lnrand(maxms-minms);
		if(Debug)
			fprint(2, "%s sleep=%ld\n", argv0, msec);
		sleep(msec);
	}
}
