#include <u.h>
#include <libc.h>
#include <bio.h>
#include <xml.h>
#include "dat.h"
#include "fns.h"

static State Album = { "album",	"" };
static State Title = { "title",	"" };
static State Artist = { "artist",	"" };
static State Length = { "length",	"" };
static State Station = { "station",	"" };
static State Cover = { "cover",	"" };

/*
 * Example playlist xml_ML live365.
 * http://www.live365.com/pls/front?handler=playlist&cmd=view&viewType=xml&handle=capitaljazz
 * NB below is one song and one advert.
 *
 * <?xml version='1.0' encoding='utf-8' standalone='yes'?>
 * <Playlist>
 *     <Refresh> 199 </Refresh>
 *     <Broadcaster> capitaljazz </Broadcaster>
 *     <PlaylistEntry>
 *         <Title> Off the Wall </Title>
 *         <Artist> json_ackiem json_oyner </Artist>
 *         <Album> json_ackiem json_oyner </Album>
 *         <ECommerceURL/>
 *         <FileName> 04_Off_the_Wall.mp3 </FileName>
 *         <trackType/>
 *         <desc/>
 *         <clickThruURL/>
 *         <visualURL> id=11994848|img=http://ecx.images-amazon.com/images/I/51VVu-tFRxL._SL130_.jpg </visualURL>
 *         <Seconds> 253 </Seconds>
 *     </PlaylistEntry>
 *     <PlaylistEntry>
 *         <Title> NONE </Title>
 *         <Artist/>
 *         <Album/>
 *         <ECommerceURL/>
 *         <FileName> Show_and_Tell_02240316_128_44_jst.mp3 </FileName>
 *         <trackType> ad </trackType>
 *         <desc> Discover Nature </desc>
 *         <clickThruURL> http://www.live365.com/ais/clickthru?streamID=94120035&amp;insertID=23584&amp;scope=ext </clickThruURL>
 *         <visualURL> id=23584|img=http://ads.live365.com/sponsors/psa/adc_fcn_getaway_300x300.jpg </visualURL>
 *         <Seconds> 30 </Seconds>
 *     </PlaylistEntry>
 * </Playlist>
 */

static char *
cover(char *str)
{
	char *p;

	if((p = strstr(str, "http")) == nil)
		return nil;
	return p;
}

void
poll(Sess *s, char *url)
{
	int fd;
	Xml *xp;
	Elem *ep;
	long msec;

	msec = 60*1000;
	for(;;){
		if((fd = httpredial(s, url, nil, 0)) < 0){
			if(! notmodified())
				fprint(2, "%s: %s httpredial failed, %r\n", argv0, url);
			goto again;
		}

		if((xp = xml_parse(fd, 1024*10, Fcrushwhite|Fstripnamespace)) == nil){
			fprint(2, "parse of last.fm xml_ML failed\n");
			close(fd);
			goto again;
		}
		close(fd);

		if((ep = xml_find(xp, xp->root, "Playlist/Refresh")) != nil){
			if(ep->pcdata){
				msec = atol(ep->pcdata);
				if(msec < 10000 || msec > 60000*2)
					msec = 60000;
			}
		}

		if((ep = xml_find(xp, xp->root, "Playlist/Broadcaster")) != nil)
			upd(&Station, ep->pcdata);

		if((ep = xml_find(xp, xp->root, "Playlist/PlaylistEntry/Title")) != nil){
			if(ep->pcdata && strcmp(ep->pcdata, "NONE") != 0)
				upd(&Title, ep->pcdata);
			else
			if((ep = xml_find(xp, xp->root, "Playlist/PlaylistEntry/desc")) != nil)
				upd(&Title, ep->pcdata);
		}

		if((ep = xml_find(xp, xp->root, "Playlist/PlaylistEntry/Artist")) != nil){
			if(ep->pcdata && strcmp(ep->pcdata, "NONE") != 0)
				upd(&Artist, ep->pcdata);
			else
			if((ep = xml_find(xp, xp->root, "Playlist/PlaylistEntry/trackType")) != nil)
				if(ep->pcdata && strcmp(ep->pcdata, "ad") == 0)
					upd(&Artist, "Advertising");
		}

		if((ep = xml_find(xp, xp->root, "Playlist/PlaylistEntry/Album")) != nil)
			upd(&Album, ep->pcdata);

		if((ep = xml_find(xp, xp->root, "Playlist/PlaylistEntry/visualURL")) != nil)
			upd(&Cover, cover(ep->pcdata));

		if((ep = xml_find(xp, xp->root, "Playlist/PlaylistEntry/Seconds")) != nil)
			upd(&Length, ep->pcdata);

		xml_free(xp);
		Bflush(&Out);
again:
		sleep(msec/2 + lnrand(msec));
	}
}

