#include <u.h>
#include <libc.h>
#include <bio.h>
#include "dat.h"
#include "fns.h"

static State Show = { "show",	"" };
static State Title = { "title",	"" };
static State Artist = { "artist",	"" };
static State Disp = { "disp",	"" };
static State Length = { "length",	"" };

/*
 * Heuristic name capitalisation, capitalise the first word
 * only (song title), or capitialise every word (artist's name)
 */
char *
capitalise(char *str, int firstonly)
{
	Rune r;
	int intra;
	char *i, *o;

	i = str;
	o = str;
	intra = 1;

	while(*i){
		i += chartorune(&r, i);
		if(intra && !islowerrune(r))
			r = toupperrune(r);
		else
		if(isupperrune(r))
			r = tolowerrune(r);
		o += runetochar(o, &r);
		if(intra && firstonly)
			break;
		intra = isspacerune(r);
	}

	while(*i){
		i += chartorune(&r, i);
		if(isupperrune(r))
			r = tolowerrune(r);
		o += runetochar(o, &r);
	}
	return str;
}

/*
 * sadly TSFJazz does not support eTags, however we try 
 * and use them anyway in case they change their mind.
 *
 * metadata seems to be clipped to 64 bytes and is allways
 * forced to upper case.
 */
void
poll(Sess *s, char *url)
{
	int n, fd;
	long msec;
	char *p, etag[1024], buf[128];

	msec = 10*1000;
	*etag = 0;
	for(;;){
		if((fd = httpredial(s, url, etag, sizeof(etag))) < 0){
			if(! notmodified())
				fprint(2, "%s: httpredial failed, %r\n", argv0);
			goto again;
		}
		buf[0] = 0;
		if((n = read(fd, buf, sizeof(buf))) >= 0)
			buf[n] = 0;
		close(fd);
		if((p = strchr(buf, '|')) != nil){
			*p = 0;
			upd(&Title, capitalise(p+1, 0));
		}
		upd(&Artist, capitalise(buf, 1));
		Bflush(&Out);
again:
		sleep(msec/2 + lnrand(msec));
	}
}
